# xDevs.com Python 10V test for 3458A
# http://xdevs.com/guide/ni_gpib_rpi/
import os.path
import sys
import Gpib
import time
import ftplib
import numbers
import signal
from Adafruit_BME280 import *

root_dir = '/repo/3458/'
fileName4 = root_dir + 'time_100madc_3458_nplc100_tin_ng_00000.csv'
fileName5 = root_dir + 'time_100madc_2000_nplc10_tin_ng_00000.csv'
fileName6 = root_dir + 'time_100madc_2003_nplc50_tin_ng_00000.csv'

cnt = 0
tread = 20
reflevel = 0.1
refhp = 0.1

rtd_temp = 0.0
tec_rtd = 0.3
temp = 18
res_rtd = 0.3
exttemp = 25.0
rh = 50.0
pascals = 100000.00
hectopascals = 1000.00
tset = 18.0

class Timeout():
  """Timeout class using ALARM signal"""
  class Timeout(Exception): pass

  def __init__(self, sec):
    self.sec = sec

  def __enter__(self):
    signal.signal(signal.SIGALRM, self.raise_timeout)
    signal.alarm(self.sec)

  def __exit__(self, *args):
    signal.alarm(0) # disable alarm

  def raise_timeout(self, *args):
    raise Timeout.Timeout()

class tec_meter():
    temp = 38.5
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global tec_rtd
    global rtd_temp

    def __init__(self,gpib,reflevel,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=60) # SCPI GPIB Address = self.gpib
        self.reflevel = reflevel
        self.name = name
	self.tec_rtd = tec_rtd
        self.init_inst()

    def init_inst(self):
        # Setup SCPI DMM
	#self.inst.write("*rst") #reset TEC controller
	#self.inst.write("*CLR")
	#Set temperature transducer to thermistor and 4-wire sense mode
	self.inst.write(":sens:temp:tran rtd")      #select thermistor
	self.inst.write(":sens:temp:rtd:type user") #10 kOhm thermistor
	self.inst.write(":sens:temp:rtd:alph 0.00375") #10 kOhm thermistor
	self.inst.write(":sens:temp:rtd:beta 0.160") #10 kOhm thermistor
	self.inst.write(":sens:temp:rtd:delt 1.605") #10 kOhm thermistor
	self.inst.write(":sens:temp:rtd:rang 1000") #10 kOhm thermistor
	self.inst.write(":sens:temp:curr 8.333e-4") #10 kOhm thermistor
	self.inst.write(":syst:rsen on")            #4-wire mode enabled
	#Set voltage limit
	self.inst.write(":sour:volt:prot 10.0") #+9.5V limit
	#Set current limit
	self.inst.write(":sens:curr:prot 5.0") #2.5A limit
	#Set temperature limit*
	self.inst.write(":sour:temp:prot:high 65")  #75C max temp
	self.inst.write(":sour:temp:prot:low 15")   #15C min temp
	self.inst.write(":sour:temp:prot:state ON") #enable temp #protection (default)
        #self.inst.write(":FORM:ELEM READ")
        self.inst.write(":DISP:WIND:TEXT:DATA \"               \";STAT OFF;")
        self.inst.write(":DISP:WIND2:TEXT:DATA \"          \";STAT OFF;")
        #kei.write("READ?")

    def cfg_temp(self):
	self.inst.write(":sour:temp 45") #set temp
	time.sleep(0.1)
        self.inst.write(":OUTP OFF")
	time.sleep(0.1)
	tec_rtd = 5.3

    def deduct_tmp(self,tmp):
	string = tmp
	print ("Setting %d" % string)
	self.inst.write(":sour:temp %2d" % string) 
	time.sleep(0.1)
        self.inst.write(":OUTP OFF")
	time.sleep(0.1)

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        time.sleep(0.1)
	try:
            with Timeout(20):
		time.sleep(0.1)
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_data(self):
        self.status_flag,data = self.read_data(":MEAS:TEMP?")
    	time.sleep(0.1)
        tec_rtd = float(data)
        if (self.status_flag):
            self.data = data
            self.ppm = ((float(self.data) / self.reflevel)-1)*1E6
    	return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self,fileHandle):
	global rtd_temp
	print ("TEC TC:%2.3f " % (float(self.data) ) )
	tec_rtd = float(self.data)
	rtd_temp = tec_rtd
	#fileHandle.write("%2.3f;\r\n" % tec_rtd);
        #print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;31m[%8d]: %2.8f , dev %4.2f ppm,\033[1;39m  EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa" % (cnt, float(self.data),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) )
        #fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%16.8f;%16.8f;%3.1f;%3.2f;%3.2f;%4.2f;\r\n" % (float(self.data),float(self.reflevel),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) ))

    def print_ppm(self):
        #self.inst.write(":DISP:WIND2:TEXT:DATA \"%3.3f ppm\"" % float(self.ppm))
	global rtd_temp
	tec_rtd = float(self.data)
	rtd_temp = tec_rtd
	#print ("%2.3f" % tec_rtd )
	#self.inst.write(":DISP:WIND2:TEXT:DATA \"  \"")


class scpi_meter():
    temp = 38.5
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global res_rtd
    global rtd_temp

    def __init__(self,gpib,reflevel,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=60) # SCPI GPIB Address = self.gpib
        self.reflevel = reflevel
        self.name = name
        if (gpib == 16):
	    self.init_instp()
	else:
            self.init_inst()

    def init_inst(self):
        # Setup SCPI DMM
        self.inst.clear()
	self.inst.write("*RST")
	self.inst.write("*CLR")
        self.inst.write(":SYST:AZER:TYPE SYNC")
        self.inst.write(":SYST:LSYN:STAT ON")
	#self.inst.write(":sens:temp:tran rtd")      #select thermistor
	#self.inst.write(":sens:temp:rtd:type user") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:alph 0.00375") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:beta 0.160") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:delt 1.605") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:rzer 1000") #10 kOhm thermistor
	self.inst.write(":SENS:FUNC 'CURR:DC'")
        self.inst.write(":SENS:CURR:DC:DIG 7;NPLC 10;AVER:COUN 5; TCON REP")
        self.inst.write(":SENS:CURR:DC:AVER:STAT OFF")
        self.inst.write(":SENS:CURR:DC:RANG 1E-1")
        self.inst.write(":FORM:ELEM READ")
        #self.inst.write(":DISP:WIND:TEXT:DATA \"               \";STAT ON;")
        #self.inst.write(":DISP:WIND2:TEXT:DATA \"               \";STAT ON;")
        #self.inst.write("READ?")

    def init_instp(self):
        # Setup SCPI DMM
        self.inst.clear()
	self.inst.write("*RST")
	self.inst.write("*CLR")
        self.inst.write(":SYST:AZER:TYPE SYNC")
        self.inst.write(":SYST:LSYN:STAT ON")
	#self.inst.write(":sens:temp:tran rtd")      #select thermistor
	#self.inst.write(":sens:temp:rtd:type user") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:alph 0.00375") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:beta 0.160") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:delt 1.605") #10 kOhm thermistor
	#self.inst.write(":sens:temp:rtd:rzer 1000") #10 kOhm thermistor
	self.inst.write(":SENS:FUNC 'CURR:DC'")
        self.inst.write(":SENS:CURR:DC:DIG 9;NPLC 50;AVER:COUN 5; TCON REP")
        self.inst.write(":SENS:CURR:DC:AVER:STAT OFF")
        self.inst.write(":SENS:CURR:DC:RANG 1E-1")
        self.inst.write(":FORM:ELEM READ")
        self.inst.write(":DISP:WIND:TEXT:DATA \"               \";STAT ON;")
        self.inst.write(":DISP:WIND2:TEXT:DATA \"               \";STAT ON;")
        #self.inst.write("READ?")

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        try:
            with Timeout(20):
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_data(self):
        self.status_flag,data = self.read_data("READ?")
        if (self.status_flag):
            self.data = data
            self.ppm = ((float(self.data) / self.reflevel)-1)*1E6
	    self.res_rtd = data
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self, fileHandle):
	global rtd_temp
	#res_rtd = self.data
	#fileHandle.write(";%2.3f;" % res_rtd);
        print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;31m[%8d]: %2.12f , dev %4.2f ppm,\033[1;39m  EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa, RTD: %5.3f" % (cnt, float(self.data),float(self.ppm),float(exttemp),float(rh),float(hectopascals), float(rtd_temp) ) )
        fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%16.12f;%16.8f;%3.1f;%3.2f;%3.2f;%4.2f;%5.3f;\r\n" % (float(self.data),float(self.reflevel),float(self.ppm),float(exttemp),float(rh),float(hectopascals),float(rtd_temp) ) ))

    def write_datae(self, fileHandle):
	fileHandle.write("\r\n");

    def print_ppm(self):
	res_rtd = self.data
        #print ("%2.3f" % float(res_rtd))
	#self.inst.write(":DISP:WIND2:TEXT:DATA \"%3.3f ppm\"" % float(self.ppm))
	#self.inst.write(":DISP:WIND:TEXT:DATA \"  \"")


class meter():
    temp = 38.5
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global res_rtd
    global tec_rtd

    def __init__(self,gpib,refhp,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=60) # 3458A GPIB Address = self.gpib
        self.refhp = refhp
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup HP 3458A
        self.inst.clear()
        self.inst.write("PRESET NORM")
        self.inst.write("OFORMAT ASCII")
        self.inst.write("DCI 1E-1")
        self.inst.write("TARM HOLD")
        self.inst.write("TRIG AUTO")
        self.inst.write("NPLC 100")
        self.inst.write("AZERO ON")
	#self.inst.write("OCOMP ON")
        self.inst.write("LFILTER ON")
        self.inst.write("NRDGS 1,AUTO")
        self.inst.write("MEM OFF")
        self.inst.write("END ALWAYS")
        self.inst.write("NDIG 9")
	#self.inst.write("DELAY 1") # 2 second delay to mitigate OCOMP accuracy issue due DA

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
    	time.sleep(0.1)
        try:
            with Timeout(20):
		time.sleep(0.1)
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_temp(self):
        self.inst.write("TARM SGL,1")
	time.sleep(0.1)        
	self.temp_status_flag,temp = self.read_data("TEMP?")
	time.sleep(0.1)
        if (self.temp_status_flag):
            self.temp = temp
        return self.temp

    def get_temp_status(self):
        return self.temp_status_flag

    def get_data(self):
        self.status_flag,data = self.read_data("TARM SGL,1")
	time.sleep(0.1)
        if (self.status_flag):
            self.data = data
            self.ppm = ((float(self.data) / self.refhp)-1)*1E6
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self,fileHandle):
	global rtd_temp
        print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;36m[%8d]: %2.12f , dev %4.4f ppm, T:%3.1f\033[1;39m , EXT_T:%3.2f , RH:%3.2f , P:%4.2f hPa; RTD:%5.3f" % (cnt, float(self.data),float(self.ppm),float(self.temp),float(exttemp),float(rh),float(hectopascals),float(rtd_temp) ) )
        fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%16.12f;%16.8f;%3.1f;%4.3f;%3.2f;%3.2f;%4.2f;%5.3f;\r\n" % (float(self.data),float(self.refhp),float(self.temp),float(self.ppm),float(exttemp),float(rh),float(hectopascals),float(rtd_temp) ) ))

    def print_ppm(self):
        #self.inst.write("DISP OFF, \"%3.3f ppm\"" % float(self.ppm))
	self.inst.write("DISP OFF, \" \"")
	#self.inst.write("DISP OFF, \"%9.8f VREF \"" % float(self.data))

# Read Temperature, Humidity, and Barometric Pressure from BME280
def get_THP():
    global exttemp
    global rh
    global pascals
    global hectopascals
    exttemp = sensor.read_temperature()
    rh = sensor.read_humidity()
    pascals = sensor.read_pressure()
    hectopascals = pascals / 100
    time.sleep(0.1)

# Check if file exists, if not create it and add header
def create_local_file(fileName):
    if (os.path.isfile(fileName) == False):
        with open(fileName, 'a') as o:
            o.write("date;hp3458a;level;temp;ppm_level;ext_temp;rh;pressure;rtd_temp;\r\n")
            print ("file %s does not exist\r\n" % fileName) 
    else: 
        print ("file %s exists\r\n" % fileName)

meter1 = meter(3,refhp,"3458A_00000") # Gpib 3
meter2510 = tec_meter(25,reflevel,"2510_00000") # Gpib 25
meter2002 = scpi_meter(20,reflevel,"2002_00000") # Gpib 10
meter2003 = scpi_meter(16,reflevel,"2002_00000") # Gpib 10

#gen = Gpib.Gpib(0,2, timeout=60) # 3245A GPIB Address 2
#gen.clear()
#gen.write("DISP MSG,\"            \"") # Clear screen
#gen.write("DISP ON")

meter1.get_temp()
#meter2001.get_temp()
#meter3.get_temp()

#Setup temp/humidity/pressure sensor BME280
# OSAMPLE = oversampling mode
sensor = BME280(mode=BME280_OSAMPLE_8)
get_THP() # Read Temp, RH, Pressure from sensor

# Create file for 1st 3458A and write header if needed
create_local_file(fileName4)
create_local_file(fileName5)
create_local_file(fileName6)
meter2510.cfg_temp()

temp = 50

meter2510.get_data()
meter2510.print_ppm()
if (meter2510.get_data_status()):
    meter2510.print_ppm()

#if (temp < 50):
#meter2510.inst.write(":OUTP OFF")
#meter2510.write(":DISP:WIND:TEXT:DATA \"ALL Done!\";STAT ON;")
#time.sleep(0.1)
#temp = 50
#print ("Set Temp = %d" % temp)
#meter2510.deduct_tmp(temp)
#time.sleep(0.1)

meter1.get_data()
meter2002.get_data()
meter1.get_data()
meter2003.get_data()
#meter1.get_data()
#meter2002.get_data()
#meter1.get_data()
#meter2002.get_data()

while cnt <= 30000:
    cnt+=1

    with open(fileName4, 'a') as o1, open(fileName5, 'a') as o2, open(fileName6, 'a') as o3: # Open file handles for both files
        tread = tread - 1
        if (tread == 0):
            tread = 30
            meter1.get_temp()
            #meter2.get_temp()
            #meter3.get_temp()

        get_THP() # Read Temp, RH, Press from sensor
	        
	meter1.get_data()
	if (meter1.get_data_status()):
	    meter1.print_ppm()
    	    meter1.write_data(o1)
	
        #meter2001.get_data()
        #if(meter2001.get_data_status()):
        #meter2001.print_ppm()
        #    meter2001.write_data(o2)

        meter2002.get_data()
        if(meter2002.get_data_status()):
    	    meter2002.print_ppm()
	    meter2002.write_data(o2)
	
	meter2003.get_data()
        if(meter2003.get_data_status()):
    	    meter2003.print_ppm()
	    meter2003.write_data(o3)

        meter2510.get_data()
        meter2510.print_ppm()
	if (meter2510.get_data_status()):
	    meter2510.print_ppm()
    	    meter2510.write_data(o1)

    
        o1.close()
        o2.close()
        o3.close()
